const { ipcRenderer, remote } = require("electron");
const { loadTempConfiguration, loadConfiguration, saveTempConfiguration } = require("../services/config-store");
const ErrorCodes = require("../services/error-codes");

const $downloadPath = document.getElementById("downloadPath");
const $folderContainer = $downloadPath.parentNode;
const $settingsScreen = document.getElementById("settings-screen");
const $settingsForm = document.getElementById("settings");
const $cancelButton = document.getElementById("cancel-settings");
const $status = document.getElementById("status");
const $statusText = $status.querySelector(".text");

const currentConfig = loadConfiguration();
const hasConfigValues = Object.keys(currentConfig).length > 0;

$settingsForm.addEventListener("submit", sendForm);
$cancelButton.addEventListener("click", refillFields);
$folderContainer.addEventListener("click", selectDirectory);
$downloadPath.addEventListener("focus", selectDirectory);
$settingsForm.addEventListener("change", showOrHideCancelButton);

autoFillFields(loadTempConfiguration());
showOrHideCancelButton();
showToastIfNecessary();

ipcRenderer.send("set-window-height", document.body.clientHeight);

function hideStatus(e) {
  e.preventDefault();
  e.stopPropagation();
  $status.classList.add("hidden");
  $settingsScreen.classList.remove("show-status");
  document.body.removeEventListener("click", hideStatus);
}

function showToastIfNecessary() {
  const params = new URL(document.location).searchParams;
  const errorCode = params.get("err");
  if (errorCode) {
    document.body.addEventListener("click", hideStatus);
    const error = ErrorCodes[errorCode];
    $statusText.innerHTML = error.message;
    $settingsScreen.classList.add("show-status");
    $status.classList.remove("hidden");
  }
}

function refillFields(ev) {
  ev.preventDefault();
  ev.stopPropagation();

  const config = loadConfiguration();
  autoFillFields(config);
  showOrHideCancelButton();
  if (hasConfigValues) {
    saveTempConfiguration(config);
    location.replace("index.html");
  }
}

function autoFillFields(config) {
  if (config.server) {
    document.getElementById("server").value = config.server;
  }
  if (config.username) {
    document.getElementById("username").value = config.username;
  }
  if (config.password) {
    document.getElementById("password").value = config.password;
  }
  if (config.downloadPath) {
    document.getElementById("downloadPath").value = config.downloadPath;
    $folderContainer.classList.add("has-value");
  }
  if (config.interval) {
    document.getElementById("interval").value = config.interval;
  }
}

function selectDirectory(e) {
  e.preventDefault();
  e.stopPropagation();
  $folderContainer.classList.add("selected");
  setTimeout(() => {
    const dirname = remote.dialog.showSaveDialog(remote.getCurrentWindow(), { defaultPath: "einsatzliste.pdf" });
    const inputEvent = new InputEvent("input");
    $downloadPath.dispatchEvent(inputEvent);
    if (dirname) {
      $downloadPath.value = dirname;
      $folderContainer.classList.add("has-value");
    }
    setTimeout(() => {
      $folderContainer.classList.remove("selected");
    }, 200);
  }, 200);
}

function sendForm(e) {
  e.preventDefault();

  const formData = getFormData();

  if (formData.errors.length > 0) {
    formData.errors.forEach($input => {
      $input.classList.add("error");
      $input.addEventListener("input", removeErrorListener($input), { once: true });
    });
  } else {
    saveTempConfiguration(formData.fields);
    location.replace("index.html");
  }

  return;

  function removeErrorListener($element) {
    return () => {
      $element.classList.remove("error");
    };
  }
}

function getFormData() {
  const $inputs = document.querySelectorAll("#settings input");
  const formData = [].reduce.call(
    $inputs,
    (acc, input) => {
      if (input.value !== "") {
        acc.fields[input.name] = input.value;
      } else {
        acc.errors.push(input);
      }

      return acc;
    },
    { fields: {}, errors: [] }
  );
  return formData;
}

function showOrHideCancelButton() {
  const formData = getFormData();
  const allTheSame = Object.keys(formData.fields).every(key => {
    return formData.fields[key] === currentConfig[key];
  });

  const hideCancelButton = !hasConfigValues || allTheSame;

  if (hideCancelButton) {
    $cancelButton.classList.add("hidden");
  } else {
    $cancelButton.classList.remove("hidden");
  }
}
