const fs = require("fs");
const { promisify } = require("util");
const request = require("superagent");
const ErrorCodes = require("./error-codes");

async function downloader({ base, downloadPath, username, password }) {
  if (!base) {
    throw new Error("Missing 'base' option: Please provide 'https://<customer>.dispolive.de'.");
  }

  if (!username || !password) {
    throw new Error("Missing credentials: Please provide 'username' and 'password' fields.");
  }

  if (!downloadPath) {
    throw new Error(
      "Missing 'downloadPath' option: Please provide '/your/path/to/result.pdf' where to download the file."
    );
  }

  /*
  Note about superagent and cookie handling:
  It seems like multiple "Set-Cookie" headers are not supported. Saving and replaying the cookies manually from the
  login POST request into the GET request are a mandatory workaround.
  */
  const agent = request.agent();
  try {
    const userResponse = await agent.post(`${base}/login`).send({ username, password });
    const cookies = userResponse.headers["set-cookie"];

    return new Promise((resolve, reject) => {
      const tempFileName = `${downloadPath}.tmp`;
      const tempFile = fs.createWriteStream(tempFileName);
      agent
        .get(`${base}/print/fahrberichte/einsatzliste.pdf`)
        .on("error", error => reject({ internalCode: ErrorCodes.COULD_NOT_DOWNLOAD, error }))
        .set("Cookie", cookies)
        .pipe(tempFile)
        .on("error", error => reject({ internalCode: ErrorCodes.COULD_NOT_SAVE, error }))
        .on("close", async () => {
          try {
            await promisify(fs.unlink)(downloadPath);
          } catch (error) {
            if (error.code !== "ENOENT") {
              return reject({ internalCode: ErrorCodes.COULD_NOT_REPLACE, error });
            }
          }
          try {
            await promisify(fs.rename)(tempFileName, downloadPath);
          } catch (error) {
            return reject({ internalCode: ErrorCodes.RENAME_FAILED, error });
          }
          return resolve(downloadPath);
        });
    });
  } catch (error) {
    throw { internalCode: ErrorCodes.COULD_NOT_LOGIN, error };
  }
}

module.exports = downloader;
